# Created by Jan Rummens at 19/01/2021
import unittest

from nemonet.engines.reporter import Reporter

class ReporterTestCase(unittest.TestCase):

    def test_zephyr(self):
        reporter = Reporter()
        reporter.publish("dummy_zephyr", Reporter.status_failed)


if __name__ == '__main__':
    unittest.main()



import threading
import time
import subprocess
import pyautogui
import os
import glob
import cv2

from .store import Store


class ScreenRecorder( object ):
    def __init__(self, interval=1):
        self.interval = interval
        self.aThread = threading.Thread(target=self.take_screenshot )
        self.halt = False
        self.store = Store()

    def take_screenshot(self):
        while not self.halt:
            file_name = self.store.generate_file()
            pyautogui.screenshot( file_name )
            time.sleep( self.interval )

    def start(self):
        self.aThread.start()

    def stop(self):
        self.halt = True
        self.aThread.join()
        self.make_video()
        self.store.zip()

    def make_video(self):
        ffmpeg_glob = self.store.get_ffmpeg_glob()
        video_file_name_prefix = (os.path.join(self.store.get_dir_name(), self.store.get_guid()))
        video_file_name = '%s-video.avi' % (video_file_name_prefix)
        fourcc = cv2.VideoWriter_fourcc(*'XVID')
        nbr=0
        height = width = layers = 0
        video=None

        for file in glob.glob(ffmpeg_glob):
            if nbr == 0:
                frame = cv2.imread(file)
                height, width, layers = frame.shape
                video = cv2.VideoWriter(video_file_name, fourcc, 2, (width, height), True)
                video.write(cv2.imread(file))
            else:
                video.write(cv2.imread(file))

            nbr+=1

        video.release()
        self.store.add_file( video_file_name )


# Created by Jan Rummens at 8/01/2021
from nemonet.runner.vision_runner import Runner
from nemonet.engines.reporter import Reporter
import typer
import sys
import traceback

app = typer.Typer()

@app.command()
def scenario(name: str, useconfig: bool = False):
    try:
        if useconfig:
            runner = Runner(runner_config="runner_config.json")
        else:
            runner = Runner()
        runner.execute_scenario(name)
    except ValueError:
        typer.echo(f"invalid commandline")
    except FileNotFoundError as e:
        typer.echo(e)
    except Exception as err:
        traceback.print_tb(err.__traceback__)
    finally:
        reporter = Reporter()
        if runner.scenario_passed():
            reporter.publish(name, Reporter.status_passed)
        else:
            reporter.publish(name, Reporter.status_failed)



# Created by Jan Rummens at 3/12/2020
from nemonet.engines.graph import Graph
from nemonet.engines.sequencer import Sequences
from nemonet.seleniumwebdriver.commands import Command
from nemonet.screencast.recording import ScreenRecorder
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import json
import time

selenium_config = {
    "driver": {
        "headless": False,
        "browser": "chrome",
        "type": "selenium"
    },
    "recording": {
        "switch": False
    }
}

class TestDriver:

    def open(self):
        pass

    def close(self):
        pass

class ChromeTestDriver( TestDriver ):

    def __init__(self):
        self.driver = None
        self.options = None

    def set_options(self ,options):
        self.options = options

    def open(self):
        if self.options == None:
            self.driver = webdriver.Chrome()
            self.driver.maximize_window()
        else:
            chrome_local_state_prefs = {'browser.enabled_labs_experiments': self.options["experimental_flags"] }
            chrome_options = Options()
            chrome_options.add_experimental_option('localState', chrome_local_state_prefs)
            self.driver = webdriver.Chrome(options=chrome_options)
            self.driver.maximize_window()

    def close(self):
        self.driver.close()

    def get_native_driver(self):
        return self.driver


class Runner( object ):

    def __init__( self, runner_config=None):
        self.data = None
        self.driver = None
        self.has_passed = False

        if runner_config == None:
            self.data = selenium_config
        else:
            with open(runner_config, 'r') as fp:
                self.data = json.load(fp)

        self.__configuration__()

    def __configuration__(self):
        if self.data["driver"]["type"] == "selenium":
            if self.data["driver"]["browser"] == "chrome":
                self.driver = ChromeTestDriver()
                if ('options' in self.data["driver"]) and ('experimental_flags' in self.data["driver"]["options"].keys() ):
                    self.driver.set_options( self.data["driver"]["options"] )
                self.driver.open()

    def turn_on_recording(self):
        if not self.is_recording:
            self.screenrecording.start()
            self.is_recording = True
            time.sleep(0.25)

    def turn_off_recording(self):
        if self.is_recording:
            self.screenrecording.stop()
            time.sleep(0.25)

    def scenario_passed(self):
        return self.has_passed

    def execute_scenario(self, xml_files_name):
        if self.data["recording"]['switch']:
            self.is_recording = False
            self.screenrecording = ScreenRecorder()
            self.turn_on_recording()
        graph = Graph()
        graph.build(xml_files_name)
        seqences = Sequences(graph)
        commands = Command(self.driver.get_native_driver())
        commands.executeSequences(seqences, graph)
        self.driver.close()
        self.has_passed = True



import os
from PIL import Image
import time

class PageCapturing:

    def __init__( self, selenium_driver):
        self.selenium_driver = selenium_driver

    def prepare_page(self):
        pass

    def capture_save( self, file_name_cpatured='capture_save_test.png'):
        self.selenium_driver.execute_script('document.body.style.overflow = "hidden";')

        total_width = self.selenium_driver.execute_script("return document.body.offsetWidth")
        total_height = self.selenium_driver.execute_script("return document.body.parentNode.scrollHeight")
        viewport_width = self.selenium_driver.execute_script("return document.body.clientWidth")
        viewport_height = self.selenium_driver.execute_script("return window.innerHeight")

        rectangles = []

        i = 0
        while i < total_height:
            ii = 0
            top_height = i + viewport_height

            if top_height > total_height:
                top_height = total_height

            while ii < total_width:
                top_width = ii + viewport_width

                if top_width > total_width:
                    top_width = total_width
                rectangles.append((ii, i, top_width, top_height))

                ii = ii + viewport_width

            i = i + viewport_height

        stitched_image = Image.new('RGB', (total_width, total_height))
        previous = None
        part = 0
        self.selenium_driver.execute_script("window.scrollTo(0, 0)")
        for rectangle in rectangles:
            if previous is not None:
                self.selenium_driver.execute_script("window.scrollTo({0}, {1})".format(rectangle[0], rectangle[1]))
                # print("Scrolled To ({0},{1})".format(rectangle[0], rectangle[1]))
                time.sleep(0.2)

            file_name = "part_{0}.png".format(part)
            # print("Capturing {0} ...".format(file_name))

            self.selenium_driver.get_screenshot_as_file(file_name)
            screenshot = Image.open(file_name)

            if rectangle[1] + viewport_height > total_height:
                offset = (rectangle[0], total_height - viewport_height)
            else:
                offset = (rectangle[0], rectangle[1])

            # print("Adding to stitched image with offset ({0}, {1})".format(offset[0], offset[1]))
            stitched_image.paste(screenshot, offset)

            del screenshot
            os.remove(file_name)
            part = part + 1
            previous = rectangle

        stitched_image.save( file_name_cpatured )


